/*
  JWildfire - an image and animation processor written in Java 
  Copyright (C) 1995-2011 Andreas Maschke

  This is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser 
  General Public License as published by the Free Software Foundation; either version 2.1 of the 
  License, or (at your option) any later version.
 
  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License along with this software; 
  if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jwildfire.create.tina.render;

import java.io.Serializable;

import org.jwildfire.create.tina.edit.Assignable;

public class RenderInfo implements Assignable<RenderInfo>, Serializable {
  private static final long serialVersionUID = 1L;
  private boolean renderHDR;
  private boolean renderHDRIntensityMap;
  private int imageWidth;
  private int imageHeight;

  protected RenderInfo() {

  }

  public RenderInfo(int pImageWidth, int pImageHeight) {
    imageWidth = pImageWidth;
    imageHeight = pImageHeight;
  }

  public boolean isRenderHDR() {
    return renderHDR;
  }

  public void setRenderHDR(boolean renderHDR) {
    this.renderHDR = renderHDR;
  }

  public boolean isRenderHDRIntensityMap() {
    return renderHDRIntensityMap;
  }

  public void setRenderHDRIntensityMap(boolean renderHDRIntensityMap) {
    this.renderHDRIntensityMap = renderHDRIntensityMap;
  }

  public int getImageWidth() {
    return imageWidth;
  }

  public void setImageWidth(int imageWidth) {
    this.imageWidth = imageWidth;
  }

  public int getImageHeight() {
    return imageHeight;
  }

  public void setImageHeight(int imageHeight) {
    this.imageHeight = imageHeight;
  }

  @Override
  public void assign(RenderInfo pSrc) {
    renderHDR = pSrc.renderHDR;
    renderHDRIntensityMap = pSrc.renderHDRIntensityMap;
    imageWidth = pSrc.imageWidth;
    imageHeight = pSrc.imageHeight;
  }

  @Override
  public RenderInfo makeCopy() {
    RenderInfo res = new RenderInfo();
    res.assign(this);
    return res;
  }

  @Override
  public boolean isEqual(RenderInfo pSrc) {
    if (renderHDR != pSrc.renderHDR || renderHDRIntensityMap != pSrc.renderHDRIntensityMap ||
        imageWidth != pSrc.imageWidth || imageHeight != pSrc.imageHeight) {
      return false;
    }
    return true;
  }

}
